"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Guard = void 0;
const utils_1 = require("@typescript-eslint/utils");
class Guard {
    isTsx;
    options;
    sourceCode;
    constructor(scope) {
        this.isTsx = scope.isTsx;
        this.options = scope.options;
        this.sourceCode = scope.sourceCode;
    }
    isAnyFunction(value) {
        return [
            utils_1.AST_NODE_TYPES.FunctionDeclaration,
            utils_1.AST_NODE_TYPES.FunctionExpression,
            utils_1.AST_NODE_TYPES.ArrowFunctionExpression,
        ].includes(value.type);
    }
    isReturnStatement(value) {
        return value?.type === utils_1.AST_NODE_TYPES.ReturnStatement;
    }
    isBlockStatementWithSingleReturn(body) {
        return (body.type === utils_1.AST_NODE_TYPES.BlockStatement &&
            body.body.length === 1 &&
            this.isReturnStatement(body.body[0]) &&
            body.body[0].argument !== null);
    }
    hasImplicitReturn(body) {
        return body.type !== utils_1.AST_NODE_TYPES.BlockStatement;
    }
    returnsImmediately(fn) {
        return this.isBlockStatementWithSingleReturn(fn.body) || this.hasImplicitReturn(fn.body);
    }
    isExportedAsNamedExport(node) {
        return node.parent.type === utils_1.AST_NODE_TYPES.ExportNamedDeclaration;
    }
    getPreviousNode(fn) {
        const node = this.isExportedAsNamedExport(fn) ? fn.parent : fn;
        const tokenBefore = this.sourceCode.getTokenBefore(node);
        if (!tokenBefore)
            return null;
        return this.sourceCode.getNodeByRangeIndex(tokenBefore.range[0]);
    }
    isOverloadedFunction(fn) {
        const previousNode = this.getPreviousNode(fn);
        return (previousNode?.type === utils_1.AST_NODE_TYPES.TSDeclareFunction ||
            (previousNode?.type === utils_1.AST_NODE_TYPES.ExportNamedDeclaration &&
                previousNode.declaration?.type === utils_1.AST_NODE_TYPES.TSDeclareFunction));
    }
    hasTypeParameters(fn) {
        return Boolean(fn.typeParameters);
    }
    isAsyncFunction(node) {
        return node.async === true;
    }
    isGeneratorFunction(fn) {
        return fn.generator === true;
    }
    isAssertionFunction(fn) {
        return (fn.returnType?.typeAnnotation.type === utils_1.AST_NODE_TYPES.TSTypePredicate && fn.returnType?.typeAnnotation.asserts);
    }
    containsToken(type, value, node) {
        return this.sourceCode.getTokens(node).some((token) => token.type === type && token.value === value);
    }
    containsSuper(node) {
        return this.containsToken('Keyword', 'super', node);
    }
    containsThis(node) {
        return this.containsToken('Keyword', 'this', node);
    }
    containsArguments(node) {
        return this.containsToken('Identifier', 'arguments', node);
    }
    containsTokenSequence(sequence, node) {
        return this.sourceCode.getTokens(node).some((_, tokenIndex, tokens) => {
            return sequence.every(([expectedType, expectedValue], i) => {
                const actual = tokens[tokenIndex + i];
                return actual && actual.type === expectedType && actual.value === expectedValue;
            });
        });
    }
    containsNewDotTarget(node) {
        return this.containsTokenSequence([
            ['Keyword', 'new'],
            ['Punctuator', '.'],
            ['Identifier', 'target'],
        ], node);
    }
    isPrototypeAssignment(node) {
        return this.sourceCode
            .getAncestors(node)
            .reverse()
            .some((ancestor) => {
            const isPropertyOfReplacementPrototypeObject = ancestor.type === utils_1.AST_NODE_TYPES.AssignmentExpression &&
                ancestor.left &&
                'property' in ancestor.left &&
                ancestor.left.property &&
                'name' in ancestor.left.property &&
                ancestor.left.property.name === 'prototype';
            const isMutationOfExistingPrototypeObject = ancestor.type === utils_1.AST_NODE_TYPES.AssignmentExpression &&
                ancestor.left &&
                'object' in ancestor.left &&
                ancestor.left.object &&
                'property' in ancestor.left.object &&
                ancestor.left.object.property &&
                'name' in ancestor.left.object.property &&
                ancestor.left.object.property.name === 'prototype';
            return isPropertyOfReplacementPrototypeObject || isMutationOfExistingPrototypeObject;
        });
    }
    isWithinClassBody(node) {
        return this.sourceCode
            .getAncestors(node)
            .reverse()
            .some((ancestor) => {
            return ancestor.type === utils_1.AST_NODE_TYPES.ClassBody;
        });
    }
    isNamedFunction(fn) {
        return fn.id !== null && fn.id.name !== null;
    }
    hasNameAndIsExportedAsDefaultExport(fn) {
        return this.isNamedFunction(fn) && fn.parent.type === utils_1.AST_NODE_TYPES.ExportDefaultDeclaration;
    }
    isIgnored(fn) {
        return this.isNamedFunction(fn) && this.options.allowedNames.includes(fn.id.name);
    }
    isObjectProperty(fn) {
        return this.sourceCode
            .getAncestors(fn)
            .reverse()
            .some((ancestor) => {
            return ancestor.type === utils_1.AST_NODE_TYPES.Property;
        });
    }
    isSafeTransformation(fn) {
        const isSafe = this.isAnyFunction(fn) &&
            !this.isGeneratorFunction(fn) &&
            !this.isAssertionFunction(fn) &&
            !this.isOverloadedFunction(fn) &&
            !this.containsThis(fn) &&
            !this.containsSuper(fn) &&
            !this.containsArguments(fn) &&
            !this.containsNewDotTarget(fn);
        if (!isSafe)
            return false;
        if (this.isIgnored(fn))
            return false;
        if (this.options.allowNamedFunctions && this.isNamedFunction(fn))
            return false;
        if (!this.options.disallowPrototype && this.isPrototypeAssignment(fn))
            return false;
        if (this.options.singleReturnOnly && !this.returnsImmediately(fn))
            return false;
        if (this.isObjectProperty(fn) && this.options.allowObjectProperties)
            return false;
        if (this.hasNameAndIsExportedAsDefaultExport(fn))
            return false;
        return true;
    }
}
exports.Guard = Guard;
//# sourceMappingURL=guard.js.map