"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.preferArrowFunctions = void 0;
const utils_1 = require("@typescript-eslint/utils");
const config_1 = require("./config");
const guard_1 = require("./guard");
const writer_1 = require("./writer");
const createRule = utils_1.ESLintUtils.RuleCreator((name) => `https://github.com/JamieMason/${name}`);
exports.preferArrowFunctions = createRule({
    name: 'prefer-arrow-functions',
    meta: {
        type: 'suggestion',
        docs: {
            description: 'Auto-fix plain Functions into Arrow Functions, in all cases where conversion would result in the same behaviour',
        },
        fixable: 'code',
        messages: config_1.MESSAGES_BY_ID,
        schema: [
            {
                additionalProperties: false,
                properties: {
                    allowedNames: {
                        default: config_1.DEFAULT_OPTIONS.allowedNames,
                        items: {
                            type: 'string',
                        },
                        type: 'array',
                    },
                    allowNamedFunctions: {
                        default: config_1.DEFAULT_OPTIONS.allowNamedFunctions,
                        type: 'boolean',
                    },
                    allowObjectProperties: {
                        default: config_1.DEFAULT_OPTIONS.allowObjectProperties,
                        type: 'boolean',
                    },
                    classPropertiesAllowed: {
                        default: config_1.DEFAULT_OPTIONS.classPropertiesAllowed,
                        type: 'boolean',
                    },
                    disallowPrototype: {
                        default: config_1.DEFAULT_OPTIONS.disallowPrototype,
                        type: 'boolean',
                    },
                    returnStyle: {
                        default: config_1.DEFAULT_OPTIONS.returnStyle,
                        pattern: '^(explicit|implicit|unchanged)$',
                        type: 'string',
                    },
                    singleReturnOnly: {
                        default: config_1.DEFAULT_OPTIONS.singleReturnOnly,
                        type: 'boolean',
                    },
                },
                type: 'object',
            },
        ],
    },
    defaultOptions: [config_1.DEFAULT_OPTIONS],
    create: (ctx, [options]) => {
        const isTsx = ctx.physicalFilename?.endsWith('.tsx');
        const sourceCode = ctx.sourceCode;
        const scope = {
            isTsx,
            options,
            sourceCode,
        };
        const guard = new guard_1.Guard(scope);
        const writer = new writer_1.Writer(scope, guard);
        const getMessageId = (node) => {
            return options.singleReturnOnly && guard.returnsImmediately(node)
                ? 'USE_ARROW_WHEN_SINGLE_RETURN'
                : 'USE_ARROW_WHEN_FUNCTION';
        };
        return {
            'ExportDefaultDeclaration > FunctionDeclaration': (node) => {
                if (guard.isSafeTransformation(node)) {
                    ctx.report({
                        fix: (fixer) => fixer.replaceText(node, writer.writeArrowFunction(node) + ';'),
                        messageId: getMessageId(node),
                        node,
                    });
                }
            },
            ':matches(ClassProperty, MethodDefinition, Property)[key.name][value.type="FunctionExpression"][kind!=/^(get|set|constructor)$/]': (node) => {
                const fn = node.value;
                if (guard.isSafeTransformation(fn) && (!guard.isWithinClassBody(fn) || options.classPropertiesAllowed)) {
                    const name = 'name' in node.key ? node.key.name : '';
                    const propName = node.key.type === utils_1.AST_NODE_TYPES.PrivateIdentifier ? `#${name}` : name;
                    const staticModifier = 'static' in node && node.static ? 'static ' : '';
                    ctx.report({
                        fix: (fixer) => fixer.replaceText(node, guard.isWithinClassBody(node)
                            ? `${staticModifier}${propName} = ${writer.writeArrowFunction(fn)};`
                            : `${staticModifier}${propName}: ${writer.writeArrowFunction(fn)}`),
                        messageId: getMessageId(fn),
                        node: fn,
                    });
                }
            },
            'ArrowFunctionExpression[body.type!="BlockStatement"]': (node) => {
                if (options.returnStyle === 'explicit' && guard.isSafeTransformation(node)) {
                    ctx.report({
                        fix: (fixer) => fixer.replaceText(node, writer.writeArrowFunction(node)),
                        messageId: 'USE_EXPLICIT',
                        node,
                    });
                }
            },
            'ArrowFunctionExpression[body.body.length=1][body.body.0.type="ReturnStatement"]': (node) => {
                if (options.returnStyle === 'implicit' && guard.isSafeTransformation(node)) {
                    ctx.report({
                        fix: (fixer) => fixer.replaceText(node, writer.writeArrowFunction(node)),
                        messageId: 'USE_IMPLICIT',
                        node,
                    });
                }
            },
            'FunctionExpression[parent.type!=/^(ClassProperty|MethodDefinition|Property)$/]': (node) => {
                if (guard.isSafeTransformation(node)) {
                    ctx.report({
                        fix: (fixer) => {
                            return fixer.replaceText(node, writer.writeArrowFunction(node));
                        },
                        messageId: getMessageId(node),
                        node,
                    });
                }
            },
            'FunctionDeclaration[parent.type!="ExportDefaultDeclaration"]': (node) => {
                if (guard.isSafeTransformation(node)) {
                    ctx.report({
                        fix: (fixer) => fixer.replaceText(node, writer.writeArrowConstant(node) + ';'),
                        messageId: getMessageId(node),
                        node,
                    });
                }
            },
        };
    },
});
//# sourceMappingURL=rule.js.map